/*****************************************************************************
 *            Created: 8/05/2012
 *          Copyright: (C) Copyright David Goodwin, 2012
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

/* Functions for dealing with grids */

#ifndef GRID_H
#define GRID_H

#include <QString>
#include <stdint.h>

/* Original gridfile format */
#define GRIDFILE_HDR1 "ZXLGF1"

/* Newer format. Has a description field */
#define GRIDFILE_HDR2 "ZXLGF2"

typedef bool** grid_t;

/* Create a new grid of the specified size.
 */
grid_t make_grid(unsigned int width, unsigned int height);

///** Creates a new grid from a text representation.
// * @param width out - the width of the new grid
// * @param height out - the height of the new grid
// * @param description out - the grids description
// * @return A new grid.
// */
//grid_t make_grid_from_text(QString grid_data, uint32_t *width,
//                           uint32_t *height, QString *description);

/* Free the specified grid.
 */
void free_grid(grid_t grid, unsigned int width);

/* Duplicate the grid returning a new grid.
 */
grid_t duplicate_grid(grid_t grid, unsigned int width, unsigned int height);

/* Resizes the specified grid returning the newly resized grid. */
grid_t resize_grid(grid_t grid, unsigned int old_width, unsigned int old_height,
                   unsigned int new_width, unsigned int new_height);

/* Copies data from the source grid to the target. Both grids must be the
 * same size. */
void copy_grid(grid_t target, grid_t source, unsigned int width, unsigned int height);

/* Loads a grid file returning a new grid, its height and its width. */
grid_t load_grid(unsigned int *width, unsigned int *height, QString filename,
                 QString *description = 0);

/* Saves the specified grid */
void save_grid_v1(grid_t grid, uint32_t width, uint32_t height, QString filename);
void save_grid(grid_t grid, uint32_t width, uint32_t height, QString filename,
               QString description = "");

/* Reduces the grid to the minimum size without removing anything and returns
 * it as a new grid. The input grid is not modified.
 */
grid_t crop_grid(grid_t grid,
                 unsigned int width,
                 unsigned int height,
                 unsigned int *new_width,
                 unsigned int *new_height);

#endif // GRID_H
